@@ -4,12 +4,51 @@
 //! human-readable, JSON, CSV, and quiet modes.
 
 use crate::baseline::ComparisonResult;
-use crate::cli::OutputFormat;
-use crate::types::{MonitorResult, ProcessMemoryInfo};
-use anyhow::Result;
-use bytesize::ByteSize;
+use crate::cli::{MemoryUnit, OutputFormat};
+use crate::types::{ByteSize, MonitorResult, ProcessMemoryInfo, Result};
 use std::io::{self, Write};
 
+/// Simple CSV writer that handles escaping
+struct CsvWriter<W: Write> {
+    writer: W,
+}
+
+impl<W: Write> CsvWriter<W> {
+    fn new(writer: W) -> Self {
+        CsvWriter { writer }
+    }
+
+    /// Write a single CSV record (row)
+    fn write_record(&mut self, fields: &[&str]) -> Result<()> {
+        for (i, field) in fields.iter().enumerate() {
+            if i > 0 {
+                write!(self.writer, ",")?;
+            }
+            // Escape field if it contains comma, quote, or newline
+            if field.contains(',') || field.contains('"') || field.contains('\n') {
+                write!(self.writer, "\"")?;
+                for c in field.chars() {
+                    if c == '"' {
+                        write!(self.writer, "\"\"")?;
+                    } else {
+                        write!(self.writer, "{}", c)?;
+                    }
+                }
+                write!(self.writer, "\"")?;
+            } else {
+                write!(self.writer, "{}", field)?;
+            }
+        }
+        writeln!(self.writer)?;
+        Ok(())
+    }
+
+    fn flush(&mut self) -> Result<()> {
+        self.writer.flush()?;
+        Ok(())
+    }
+}
+
 /// Handles formatting of monitoring results for different output formats.
 pub struct OutputFormatter;
 
@@ -20,13 +59,19 @@ impl OutputFormatter {
     /// * `result` - The monitoring results to format
     /// * `format` - The output format to use
     /// * `verbose` - Whether to include verbose information
-    pub fn format(result: &MonitorResult, format: OutputFormat, verbose: bool) -> Result<()> {
+    /// * `units` - Optional fixed memory unit to use for display
+    pub fn format(
+        result: &MonitorResult,
+        format: OutputFormat,
+        verbose: bool,
+        units: Option<MemoryUnit>,
+    ) -> Result<()> {
         match format {
             OutputFormat::Human => {
                 if verbose {
-                    Self::format_verbose(result)
+                    Self::format_verbose(result, units)
                 } else {
-                    Self::format_human(result)
+                    Self::format_human(result, units)
                 }
             }
             OutputFormat::Json => Self::format_json(result),
@@ -35,12 +80,22 @@ impl OutputFormatter {
         }
     }
 
-    fn format_human(result: &MonitorResult) -> Result<()> {
+    fn format_human(result: &MonitorResult, units: Option<MemoryUnit>) -> Result<()> {
         let mut stdout = io::stdout();
 
         writeln!(stdout, "Command: {}", result.command)?;
-        write!(stdout, "Peak memory usage: {} (RSS)", result.peak_rss())?;
-        writeln!(stdout, " / {} (VSZ)", result.peak_vsz())?;
+
+        if let Some(unit) = units {
+            write!(
+                stdout,
+                "Peak memory usage: {} (RSS)",
+                unit.format(result.peak_rss_bytes)
+            )?;
+            writeln!(stdout, " / {} (VSZ)", unit.format(result.peak_vsz_bytes))?;
+        } else {
+            write!(stdout, "Peak memory usage: {} (RSS)", result.peak_rss())?;
+            writeln!(stdout, " / {} (VSZ)", result.peak_vsz())?;
+        }
 
         if let Some(exit_code) = result.exit_code {
             writeln!(stdout, "Exit code: {exit_code}")?;
@@ -63,9 +118,9 @@ impl OutputFormatter {
     }
 
     fn format_csv(result: &MonitorResult) -> Result<()> {
-        let mut wtr = csv::Writer::from_writer(io::stdout());
+        let mut wtr = CsvWriter::new(io::stdout());
 
-        wtr.write_record([
+        wtr.write_record(&[
             "command",
             "peak_rss_bytes",
             "peak_vsz_bytes",
@@ -75,12 +130,13 @@ impl OutputFormatter {
             "timestamp",
         ])?;
 
-        wtr.write_record([
+        let exit_code_str = result.exit_code.map_or(String::new(), |c| c.to_string());
+        wtr.write_record(&[
             &result.command,
             &result.peak_rss_bytes.to_string(),
             &result.peak_vsz_bytes.to_string(),
             &result.duration_ms.to_string(),
-            &result.exit_code.map_or("".to_string(), |c| c.to_string()),
+            &exit_code_str,
             &result.threshold_exceeded.to_string(),
             &result.timestamp.to_rfc3339(),
         ])?;
@@ -94,17 +150,13 @@ impl OutputFormatter {
         Ok(())
     }
 
-    fn format_verbose(result: &MonitorResult) -> Result<()> {
+    fn format_verbose(result: &MonitorResult, units: Option<MemoryUnit>) -> Result<()> {
         let mut stdout = io::stdout();
 
         // Header
         writeln!(stdout, "Command: {}", result.command)?;
         if let Some(start_time) = result.start_time {
-            writeln!(
-                stdout,
-                "Started: {} UTC",
-                start_time.format("%Y-%m-%d %H:%M:%S")
-            )?;
+            writeln!(stdout, "Started: {} UTC", start_time.format_datetime())?;
         }
         if let Some(pid) = result.main_pid {
             writeln!(stdout, "Process ID: {pid}")?;
@@ -113,18 +165,33 @@ impl OutputFormatter {
 
         // Memory Usage Section
         writeln!(stdout, "Memory Usage:")?;
-        writeln!(
-            stdout,
-            "  Peak RSS: {} ({} bytes)",
-            result.peak_rss(),
-            result.peak_rss_bytes
-        )?;
-        writeln!(
-            stdout,
-            "  Peak VSZ: {} ({} bytes)",
-            result.peak_vsz(),
-            result.peak_vsz_bytes
-        )?;
+        if let Some(unit) = units {
+            writeln!(
+                stdout,
+                "  Peak RSS: {} ({} bytes)",
+                unit.format(result.peak_rss_bytes),
+                result.peak_rss_bytes
+            )?;
+            writeln!(
+                stdout,
+                "  Peak VSZ: {} ({} bytes)",
+                unit.format(result.peak_vsz_bytes),
+                result.peak_vsz_bytes
+            )?;
+        } else {
+            writeln!(
+                stdout,
+                "  Peak RSS: {} ({} bytes)",
+                result.peak_rss(),
+                result.peak_rss_bytes
+            )?;
+            writeln!(
+                stdout,
+                "  Peak VSZ: {} ({} bytes)",
+                result.peak_vsz(),
+                result.peak_vsz_bytes
+            )?;
+        }
         writeln!(stdout)?;
 
         // Process Tree Section
@@ -134,7 +201,7 @@ impl OutputFormatter {
                 stdout,
                 "Process Tree: ({process_count} processes monitored)"
             )?;
-            Self::print_process_tree(&mut stdout, tree, "", true)?;
+            Self::print_process_tree(&mut stdout, tree, "", true, units)?;
         } else {
             writeln!(
                 stdout,
@@ -192,6 +259,7 @@ impl OutputFormatter {
         tree: &ProcessMemoryInfo,
         prefix: &str,
         is_last: bool,
+        units: Option<MemoryUnit>,
     ) -> Result<()> {
         // Print current process
         let connector = if is_last { "└── " } else { "├── " };
@@ -201,6 +269,12 @@ impl OutputFormatter {
             tree.name.clone()
         };
 
+        let memory_str = if let Some(unit) = units {
+            unit.format(tree.memory.rss_bytes)
+        } else {
+            ByteSize::b(tree.memory.rss_bytes).to_string()
+        };
+
         writeln!(
             stdout,
             "{}{}{} (PID: {}) - Peak: {}",
@@ -208,7 +282,7 @@ impl OutputFormatter {
             if prefix.is_empty() { "" } else { connector },
             name,
             tree.pid,
-            ByteSize::b(tree.memory.rss_bytes)
+            memory_str
         )?;
 
         // Sort children by peak RSS (descending)
@@ -230,7 +304,7 @@ impl OutputFormatter {
 
         for (i, child) in children.iter().enumerate() {
             let is_last_child = i == children.len() - 1;
-            Self::print_process_tree(stdout, child, &child_prefix, is_last_child)?;
+            Self::print_process_tree(stdout, child, &child_prefix, is_last_child, units)?;
         }
 
         Ok(())
@@ -241,52 +315,96 @@ impl OutputFormatter {
     /// # Arguments
     /// * `comparison` - The comparison results
     /// * `format` - The output format to use
-    pub fn format_comparison(comparison: &ComparisonResult, format: OutputFormat) -> Result<()> {
+    /// * `units` - Optional fixed memory unit to use for display
+    pub fn format_comparison(
+        comparison: &ComparisonResult,
+        format: OutputFormat,
+        units: Option<MemoryUnit>,
+    ) -> Result<()> {
         match format {
-            OutputFormat::Human => Self::format_comparison_human(comparison),
+            OutputFormat::Human => Self::format_comparison_human(comparison, units),
             OutputFormat::Json => Self::format_comparison_json(comparison),
             OutputFormat::Csv => Self::format_comparison_csv(comparison),
             OutputFormat::Quiet => Self::format_comparison_quiet(comparison),
         }
     }
 
-    fn format_comparison_human(comparison: &ComparisonResult) -> Result<()> {
+    fn format_comparison_human(
+        comparison: &ComparisonResult,
+        units: Option<MemoryUnit>,
+    ) -> Result<()> {
         let mut stdout = io::stdout();
 
         writeln!(stdout, "Command: {}", comparison.current.command)?;
         writeln!(stdout)?;
 
         writeln!(stdout, "Baseline vs Current:")?;
-        writeln!(
-            stdout,
-            "  Peak RSS: {} → {} ({:+.1}%)",
-            ByteSize::b(comparison.baseline.peak_rss_bytes),
-            comparison.current.peak_rss(),
-            comparison.rss_diff_percent
-        )?;
-
-        if comparison.rss_diff_bytes > 0 {
+        if let Some(unit) = units {
             writeln!(
                 stdout,
-                "  Absolute increase: {}",
-                ByteSize::b(comparison.rss_diff_bytes as u64)
+                "  Peak RSS: {} → {} ({:+.1}%)",
+                unit.format(comparison.baseline.peak_rss_bytes),
+                unit.format(comparison.current.peak_rss_bytes),
+                comparison.rss_diff_percent
             )?;
-        } else if comparison.rss_diff_bytes < 0 {
+        } else {
             writeln!(
                 stdout,
-                "  Absolute decrease: {}",
-                ByteSize::b((-comparison.rss_diff_bytes) as u64)
+                "  Peak RSS: {} → {} ({:+.1}%)",
+                ByteSize::b(comparison.baseline.peak_rss_bytes),
+                comparison.current.peak_rss(),
+                comparison.rss_diff_percent
             )?;
         }
 
+        if comparison.rss_diff_bytes > 0 {
+            if let Some(unit) = units {
+                writeln!(
+                    stdout,
+                    "  Absolute increase: {}",
+                    unit.format(comparison.rss_diff_bytes as u64)
+                )?;
+            } else {
+                writeln!(
+                    stdout,
+                    "  Absolute increase: {}",
+                    ByteSize::b(comparison.rss_diff_bytes as u64)
+                )?;
+            }
+        } else if comparison.rss_diff_bytes < 0 {
+            if let Some(unit) = units {
+                writeln!(
+                    stdout,
+                    "  Absolute decrease: {}",
+                    unit.format((-comparison.rss_diff_bytes) as u64)
+                )?;
+            } else {
+                writeln!(
+                    stdout,
+                    "  Absolute decrease: {}",
+                    ByteSize::b((-comparison.rss_diff_bytes) as u64)
+                )?;
+            }
+        }
+
         writeln!(stdout)?;
-        writeln!(
-            stdout,
-            "  Peak VSZ: {} → {} ({:+.1}%)",
-            ByteSize::b(comparison.baseline.peak_vsz_bytes),
-            comparison.current.peak_vsz(),
-            comparison.vsz_diff_percent
-        )?;
+        if let Some(unit) = units {
+            writeln!(
+                stdout,
+                "  Peak VSZ: {} → {} ({:+.1}%)",
+                unit.format(comparison.baseline.peak_vsz_bytes),
+                unit.format(comparison.current.peak_vsz_bytes),
+                comparison.vsz_diff_percent
+            )?;
+        } else {
+            writeln!(
+                stdout,
+                "  Peak VSZ: {} → {} ({:+.1}%)",
+                ByteSize::b(comparison.baseline.peak_vsz_bytes),
+                comparison.current.peak_vsz(),
+                comparison.vsz_diff_percent
+            )?;
+        }
 
         writeln!(stdout)?;
         writeln!(
@@ -319,9 +437,9 @@ impl OutputFormatter {
     }
 
     fn format_comparison_csv(comparison: &ComparisonResult) -> Result<()> {
-        let mut wtr = csv::Writer::from_writer(io::stdout());
+        let mut wtr = CsvWriter::new(io::stdout());
 
-        wtr.write_record([
+        wtr.write_record(&[
             "baseline_command",
             "baseline_rss_bytes",
             "baseline_vsz_bytes",
@@ -339,7 +457,7 @@ impl OutputFormatter {
             "regression_detected",
         ])?;
 
-        wtr.write_record([
+        wtr.write_record(&[
             &comparison.baseline.command,
             &comparison.baseline.peak_rss_bytes.to_string(),
             &comparison.baseline.peak_vsz_bytes.to_string(),
@@ -376,12 +494,16 @@ impl OutputFormatter {
 /// Uses terminal control sequences to update the display in-place.
 pub struct RealtimeDisplay {
     last_line_count: usize,
+    units: Option<MemoryUnit>,
 }
 
 impl RealtimeDisplay {
     /// Creates a new real-time display handler.
-    pub fn new() -> Self {
-        Self { last_line_count: 0 }
+    pub fn new(units: Option<MemoryUnit>) -> Self {
+        Self {
+            last_line_count: 0,
+            units,
+        }
     }
 
     /// Updates the display with current memory values.
@@ -410,8 +532,23 @@ impl RealtimeDisplay {
         }
 
         // Print new status
-        writeln!(stdout, "Current RSS: {current_rss} | Peak RSS: {peak_rss}")?;
-        writeln!(stdout, "Current VSZ: {current_vsz} | Peak VSZ: {peak_vsz}")?;
+        if let Some(unit) = self.units {
+            writeln!(
+                stdout,
+                "Current RSS: {} | Peak RSS: {}",
+                unit.format(current_rss.as_u64()),
+                unit.format(peak_rss.as_u64())
+            )?;
+            writeln!(
+                stdout,
+                "Current VSZ: {} | Peak VSZ: {}",
+                unit.format(current_vsz.as_u64()),
+                unit.format(peak_vsz.as_u64())
+            )?;
+        } else {
+            writeln!(stdout, "Current RSS: {current_rss} | Peak RSS: {peak_rss}")?;
+            writeln!(stdout, "Current VSZ: {current_vsz} | Peak VSZ: {peak_vsz}")?;
+        }
         stdout.flush()?;
 
         self.last_line_count = 2;
@@ -438,8 +575,7 @@ impl RealtimeDisplay {
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::types::MemoryUsage;
-    use chrono::Utc;
+    use crate::types::{MemoryUsage, Timestamp};
 
     #[test]
     fn test_format_quiet() {
@@ -450,7 +586,7 @@ mod tests {
             duration_ms: 1000,
             exit_code: Some(0),
             threshold_exceeded: false,
-            timestamp: Utc::now(),
+            timestamp: Timestamp::now(),
             process_tree: None,
             timeline: None,
             start_time: None,
@@ -459,12 +595,12 @@ mod tests {
         };
 
         // Quiet format should just print the RSS bytes
-        OutputFormatter::format(&result, OutputFormat::Quiet, false).unwrap();
+        OutputFormatter::format(&result, OutputFormat::Quiet, false, None).unwrap();
     }
 
     #[test]
     fn test_format_verbose() {
-        let now = Utc::now();
+        let now = Timestamp::now();
 
         // Create a sample process tree
         let child_process = ProcessMemoryInfo {
@@ -526,12 +662,12 @@ mod tests {
         };
 
         // Test verbose format - should not panic
-        OutputFormatter::format(&result, OutputFormat::Human, true).unwrap();
+        OutputFormatter::format(&result, OutputFormat::Human, true, None).unwrap();
     }
 
     #[test]
     fn test_format_verbose_no_children() {
-        let now = Utc::now();
+        let now = Timestamp::now();
 
         let result = MonitorResult {
             command: "echo test".to_string(),
@@ -549,12 +685,12 @@ mod tests {
         };
 
         // Test verbose format without process tree
-        OutputFormatter::format(&result, OutputFormat::Human, true).unwrap();
+        OutputFormatter::format(&result, OutputFormat::Human, true, None).unwrap();
     }
 
     #[test]
     fn test_count_processes() {
-        let now = Utc::now();
+        let now = Timestamp::now();
         let tree = ProcessMemoryInfo {
             pid: 1,
             name: "root".to_string(),
